@@ -22,6 +22,7 @@ mod waiter;
 
 pub use self::apiversion::ApiVersion;
 pub use self::resourceiterator::ResourceIterator;
-pub use self::types::{FlavorRef, ImageRef, ListResources, NetworkRef, PortRef,
-                      ProjectRef, Refresh, ResourceId, UserRef};
+pub use self::types::{FlavorRef, ImageRef, KeyPairRef, ListResources,
+                      NetworkRef, PortRef, ProjectRef, Refresh, ResourceId,
+                      UserRef};
 pub use self::waiter::DeletionWaiter;
