@@ -37,9 +37,15 @@ const API_VERSION_KEYPAIR_PAGINATION: ApiVersion = ApiVersion(2, 35);
 
 /// Extensions for Session.
 pub trait V2API {
+    /// Create a key pair.
+    fn create_keypair(&self, request: protocol::KeyPairCreate) -> Result<protocol::KeyPair>;
+
     /// Create a server.
     fn create_server(&self, request: protocol::ServerCreate) -> Result<Ref>;
 
+    /// Delete a key pair.
+    fn delete_keypair<S: AsRef<str>>(&self, name: S) -> Result<()>;
+
     /// Delete a server.
     fn delete_server<S: AsRef<str>>(&self, id: S) -> Result<()>;
 
@@ -121,6 +127,16 @@ const SERVICE_TYPE: &'static str = "compute";
 const VERSION_ID: &'static str = "v2.1";
 
 impl V2API for Session {
+    fn create_keypair(&self, request: protocol::KeyPairCreate)
+            -> Result<protocol::KeyPair> {
+        debug!("Creating a key pair with {:?}", request);
+        let body = protocol::KeyPairCreateRoot { keypair: request };
+        let keypair = self.request::<V2>(Method::Post, &["os-keypairs"], None)?
+            .json(&body).receive_json::<protocol::KeyPairRoot>()?.keypair;
+        debug!("Created key pair {:?}", keypair);
+        Ok(keypair)
+    }
+
     fn create_server(&self, request: protocol::ServerCreate) -> Result<Ref> {
         debug!("Creating a server with {:?}", request);
         let body = protocol::ServerCreateRoot { server: request };
@@ -130,6 +146,16 @@ impl V2API for Session {
         Ok(server)
     }
 
+    fn delete_keypair<S: AsRef<str>>(&self, name: S) -> Result<()> {
+        debug!("Deleting key pair {}", name.as_ref());
+        let _ = self.request::<V2>(Method::Delete,
+                                   &["os-keypairs", name.as_ref()],
+                                   None)?
+            .send()?;
+        debug!("Key pair {} was deleted", name.as_ref());
+        Ok(())
+    }
+
     fn delete_server<S: AsRef<str>>(&self, id: S) -> Result<()> {
         trace!("Deleting server {}", id.as_ref());
         let _ = self.request::<V2>(Method::Delete,
