@@ -15,13 +15,14 @@
 //! Key pair management via Compute API.
 
 use std::fmt::Debug;
+use std::io;
 use std::rc::Rc;
 
 use fallible_iterator::{IntoFallibleIterator, FallibleIterator};
 use serde::Serialize;
 
-use super::super::{Error, Result};
-use super::super::common::{ListResources, Refresh, ResourceId,
+use super::super::{Error, ErrorKind, Result};
+use super::super::common::{KeyPairRef, ListResources, Refresh, ResourceId,
                            ResourceIterator};
 use super::super::session::Session;
 use super::super::utils::Query;
@@ -44,6 +45,14 @@ pub struct KeyPairQuery {
     can_paginate: bool,
 }
 
+/// A request to create a key pair.
+#[derive(Clone, Debug)]
+pub struct NewKeyPair {
+    session: Rc<Session>,
+    name: String,
+    public_key: Option<String>,
+}
+
 
 impl KeyPair {
     /// Load a KeyPair object.
@@ -56,6 +65,11 @@ impl KeyPair {
         })
     }
 
+    /// Delete the key pair.
+    pub fn delete(self) -> Result<()> {
+        self.session.delete_keypair(&self.inner.name)
+    }
+
     transparent_property! {
         #[doc = "Key pair fingerprint."]
         fingerprint: ref String
@@ -141,6 +155,58 @@ impl KeyPairQuery {
     }
 }
 
+impl NewKeyPair {
+    /// Start creating a key pair.
+    pub(crate) fn new(session: Rc<Session>, name: String)
+            -> NewKeyPair {
+        NewKeyPair {
+            session: session,
+            name: name,
+            public_key: None,
+        }
+    }
+
+    /// Request creation of a key pair.
+    ///
+    /// This call fails immediately if no public_key is provided.
+    pub fn create(self) -> Result<KeyPair> {
+        let request = if let Some(public_key) = self.public_key {
+            protocol::KeyPairCreate {
+                key_type: None,  // TODO
+                name: self.name,
+                public_key: public_key
+            }
+        } else {
+            return Err(Error::new(ErrorKind::InvalidInput,
+                                  "Public key contents is required"));
+        };
+
+        let keypair = self.session.create_keypair(request)?;
+        Ok(KeyPair {
+            session: self.session,
+            inner: keypair
+        })
+    }
+
+    /// Add public key from a reader.
+    pub fn from_reader<R>(self, reader: &mut R) -> io::Result<NewKeyPair> where R: io::Read {
+        let mut s = String::new();
+        let _ = reader.read_to_string(&mut s)?;
+        Ok(self.from_string(s))
+    }
+
+    /// Add public key from a string.
+    pub fn from_string<S>(mut self, public_key: S) -> NewKeyPair where S: Into<String> {
+        self.set_string(public_key);
+        self
+    }
+
+    /// Add public key from a string.
+    pub fn set_string<S>(&mut self, public_key: S) where S: Into<String> {
+        self.public_key = Some(public_key.into());
+    }
+}
+
 impl ResourceId for KeyPair {
     fn resource_id(&self) -> String {
         self.name().clone()
@@ -174,3 +240,21 @@ impl IntoFallibleIterator for KeyPairQuery {
         self.into_iter()
     }
 }
+
+impl From<KeyPair> for KeyPairRef {
+    fn from(value: KeyPair) -> KeyPairRef {
+        KeyPairRef::new_verified(value.inner.name)
+    }
+}
+
+impl KeyPairRef {
+    /// Verify this reference and convert to an ID, if possible.
+    #[cfg(feature = "compute")]
+    pub(crate) fn into_verified(self, session: &Session) -> Result<String> {
+        Ok(if self.verified {
+            self.value
+        } else {
+            session.get_keypair(&self.value)?.name
+        })
+    }
+}
