@@ -22,7 +22,7 @@ mod servers;
 
 pub use self::base::V2 as ServiceType;
 pub use self::flavors::{Flavor, FlavorSummary, FlavorQuery};
-pub use self::keypairs::{KeyPair, KeyPairQuery};
+pub use self::keypairs::{KeyPair, KeyPairQuery, NewKeyPair};
 pub use self::protocol::{AddressType, KeyPairType, RebootType, ServerAddress,
                          ServerSortKey, ServerPowerState, ServerStatus};
 pub use self::servers::{NewServer, Server, ServerCreationWaiter, ServerNIC,
