@@ -150,6 +150,8 @@ pub struct Server {
     #[serde(deserialize_with = "common::protocol::empty_as_none", default)]
     pub image: Option<common::protocol::Ref>,
     pub name: String,
+    #[serde(default)]
+    pub metadata: HashMap<String, String>,
     pub status: ServerStatus,
     #[serde(rename = "OS-EXT-STS:power_state", default)]
     pub power_state: ServerPowerState,
@@ -188,6 +190,8 @@ pub struct ServerCreate {
     pub imageRef: Option<String>,
     #[serde(skip_serializing_if = "Option::is_none")]
     pub key_name: Option<String>,
+    #[serde(skip_serializing_if = "HashMap::is_empty")]
+    pub metadata: HashMap<String, String>,
     pub name: String,
     pub networks: Vec<ServerNetwork>
 }
@@ -237,7 +241,15 @@ pub struct FlavorRoot {
 #[derive(Clone, Debug, Deserialize)]
 pub struct KeyPair {
     pub fingerprint: String,
-    #[serde(rename = "type", default)]
+    #[serde(rename = "type", default, skip_serializing_if = "Option::is_none")]
+    pub key_type: Option<KeyPairType>,
+    pub name: String,
+    pub public_key: String,
+}
+
+#[derive(Clone, Debug, Serialize)]
+pub struct KeyPairCreate {
+    #[serde(rename = "type", skip_serializing_if = "Option::is_none")]
     pub key_type: Option<KeyPairType>,
     pub name: String,
     pub public_key: String,
@@ -248,6 +260,11 @@ pub struct KeyPairRoot {
     pub keypair: KeyPair
 }
 
+#[derive(Clone, Debug, Serialize)]
+pub struct KeyPairCreateRoot {
+    pub keypair: KeyPairCreate
+}
+
 #[derive(Clone, Debug, Deserialize)]
 pub struct KeyPairsRoot {
     pub keypairs: Vec<KeyPairRoot>
