@@ -26,7 +26,7 @@ use serde::Serialize;
 use waiter::{Waiter, WaiterCurrentState};
 
 use super::super::{Error, ErrorKind, Result, Sort};
-use super::super::common::{self, DeletionWaiter, FlavorRef, ImageRef,
+use super::super::common::{self, DeletionWaiter, FlavorRef, ImageRef, KeyPairRef,
                            ListResources, NetworkRef, PortRef, ProjectRef,
                            Refresh, ResourceId, ResourceIterator, UserRef};
 use super::super::session::Session;
@@ -81,6 +81,8 @@ pub struct NewServer {
     session: Rc<Session>,
     flavor: FlavorRef,
     image: Option<ImageRef>,
+    keypair: Option<KeyPairRef>,
+    metadata: HashMap<String, String>,
     name: String,
     networks: Vec<ServerNIC>,
 }
@@ -179,6 +181,11 @@ impl Server {
         &self.inner.name
     }
 
+    /// Get a reference to associated metadata.
+    pub fn metadata(&self) -> &HashMap<String, String> {
+        &self.inner.metadata
+    }
+
     /// Get server power state.
     pub fn power_state(&self) -> protocol::ServerPowerState {
         self.inner.power_state
@@ -480,6 +487,8 @@ impl NewServer {
             session: session,
             flavor: flavor,
             image: None,
+            keypair: None,
+            metadata: HashMap::new(),
             name: name,
             networks: Vec::new(),
         }
@@ -493,7 +502,11 @@ impl NewServer {
                 Some(img) => Some(img.into_verified(&self.session)?),
                 None => None
             },
-            key_name: None,  // TODO
+            key_name: match self.keypair {
+                Some(item) => Some(item.into_verified(&self.session)?),
+                None => None
+            },
+            metadata: self.metadata,
             name: self.name,
             networks: convert_networks(&self.session, self.networks)?
         };
@@ -535,6 +548,11 @@ impl NewServer {
         self.image = Some(image.into());
     }
 
+    /// Use this key pair for the new server.
+    pub fn set_keypair<K>(&mut self, keypair: K) where K: Into<KeyPairRef> {
+        self.keypair = Some(keypair.into());
+    }
+
     /// Add a virtual NIC with given fixed IP to the new server.
     pub fn with_fixed_ip(mut self, fixed_ip: Ipv4Addr) -> NewServer {
         self.add_fixed_ip(fixed_ip);
@@ -548,6 +566,13 @@ impl NewServer {
         self
     }
 
+    /// Use this key pair for the new server.
+    pub fn with_keypair<K>(mut self, keypair: K) -> NewServer
+            where K: Into<KeyPairRef> {
+        self.set_keypair(keypair);
+        self
+    }
+
     /// Add a virtual NIC from this network to the new server.
     pub fn with_network<N>(mut self, network: N) -> NewServer
             where N: Into<NetworkRef> {
@@ -561,6 +586,14 @@ impl NewServer {
         self.add_port(port);
         self
     }
+
+    /// Add an arbitrary key/value metadata pair.
+    pub fn with_metadata<S1, S2>(mut self, key: S1, value: S2) -> NewServer
+            where S1: Into<String>,
+                  S2: Into<String> {
+        let _ = self.metadata.insert(key.into(), value.into());
+        self
+    }
 }
 
 impl Waiter<Server, Error> for ServerCreationWaiter {
