@@ -17,6 +17,7 @@ extern crate openstack;
 extern crate waiter;
 
 use std::env;
+use std::fs::File;
 use std::sync::{Once, ONCE_INIT};
 
 use waiter::Waiter;
@@ -38,14 +39,24 @@ fn test_basic_server_ops() {
     let image_id = env::var("RUST_OPENSTACK_IMAGE").expect("Missing RUST_OPENSTACK_IMAGE");
     let flavor_id = env::var("RUST_OPENSTACK_FLAVOR").expect("Missing RUST_OPENSTACK_FLAVOR");
     let network_id = env::var("RUST_OPENSTACK_NETWORK").expect("Missing RUST_OPENSTACK_NETWORK");
+    let keypair_file_name = env::var("RUST_OPENSTACK_KEYPAIR")
+        .expect("Missing RUST_OPENSTACK_KEYPAIR");
+
+    let keypair = os.new_keypair("rust-openstack-integration")
+        .from_reader(&mut File::open(keypair_file_name)
+                     .expect("Cannot open RUST_OPENSTACK_KEYPAIR"))
+        .expect("Cannot read RUST_OPENSTACK_KEYPAIR")
+        .create().expect("Cannot create a key pair");
 
     let mut server = os.new_server("rust-openstack-integration", flavor_id)
-        .with_image(image_id).with_network(network_id)
+        .with_image(image_id).with_network(network_id).with_keypair(keypair)
+        .with_metadata("meta", "a3f955c049f7416faa7")
         .create().expect("Failed to request server creation")
         .wait().expect("Server was not created");
     assert_eq!(server.name(), "rust-openstack-integration");
     assert_eq!(server.status(), openstack::compute::ServerStatus::Active);
     assert_eq!(server.power_state(), openstack::compute::ServerPowerState::Running);
+    assert_eq!(server.metadata().get("meta"), Some(&"a3f955c049f7416faa7".to_string()));
 
     server.stop().expect("Failed to request power off")
         .wait().expect("Failed to power off");
@@ -57,4 +68,8 @@ fn test_basic_server_ops() {
 
     server.delete().expect("Failed to request deletion")
         .wait().expect("Failed to delete server");
+
+    os.get_keypair("rust-openstack-integration")
+        .expect("Cannot get key pair").delete()
+        .expect("Cannot delete key pair");
 }
